@@ -1,13 +1,14 @@
 # import os
 
 __name__ = 'ccalafiore'
-__version__ = '000.000.032'
+__version__ = '000.000.039'
 __author__ = 'Calafiore Carmelo'
 __author_email__ = 'c.calafiore@essex.ac.uk'
 __maintainer_email__ = 'c.calafiore@essex.ac.uk'
 __all__ = [
-    'array', 'Checker', 'clock', 'combinations', 'directory', 'download', 'Formatter',
-    'maths', 'plot', 'preprocessing', 'shut_down', 'stats', 'strings']
+    'array', 'Checker', 'clock', 'combinations', 'directory', 'display', 'download',
+    'Formatter', 'maths', 'mixamo', 'plot', 'preprocessing', 'shut_down', 'stats',
+    'strings']
 # __all__ = []
 # cwd = os.path.dirname(__file__)
 # format_modules = '.py'
