@@ -1,10 +1,13 @@
+from .combinations import n_conditions_to_combinations
+from . import Formatter as cc_Formatter
+from .maths import convert_to_int_or_float
 import numpy as np
-from ccalafiore.combinations import n_conditions_to_combinations
-from glob import glob
+import glob
+import math
 import numbers
-from math import ceil
 import os
-import ccalafiore.Formatter as cc_Formatter
+import csv
+import cv2
 
 
 def advanced_indexing(indexes_loose):
@@ -276,133 +279,73 @@ def transfer_n_random_samples_from_arr1_to_arr2(arr1, arr2, n_samples=1, axis=0,
     return arr1, arr2
 
 
-def shuffle_in_windows(data, n_samples_window=None, n_windows=None, dimension_shuffle=0):
+def shuffle_in_windows(data, n_samples_window=None, n_windows=None, axis=0, dtype=None):
 
     type_data = type(data)
     if type_data != np.ndarray:
         raise Exception('data must be an numpy array. '
                         'Now, the class of data is {}'.format(type_data))
 
-
-    dimension_shuffle_is_integer = isinstance(dimension_shuffle, numbers.Integral)
-    # The Numeric abstract base classes:
-    # numbers.Complex
-    # numbers.Real
-    # numbers.Integral
-    # numbers.Number
-    if not dimension_shuffle_is_integer:
-        type_dimension_shuffle = type(dimension_shuffle)
-        raise Exception('dimension_shuffle must be an integer. '
-                        'Now, the class of dimension_shuffle is {}'.format(
-            type_dimension_shuffle))
-
     shape_data = data.shape
-    n_samples_data = shape_data[dimension_shuffle]
+    n_samples_data = shape_data[axis]
     n_d_data = len(shape_data)
-    if dimension_shuffle > n_d_data:
-        raise Exception('shuffle in dimension {} can\'t be done,\n'
-                        'because data has {} dimensions and\n'
-                        'it doesn\'t have the {}th dimension.\n'
-                        'dimension_shuffle has to be an integer in the interval\n'
-                        '0 =< dimension_shuffle < n_d_data.\n'
-                        'In this case, dimension_shuffle = {} and n_d_data = {}'.format(
-            dimension_shuffle, n_d_data, dimension_shuffle, dimension_shuffle, n_d_data))
-
-
-    n_samples_window_is_integer = isinstance(n_samples_window, numbers.Integral)
-
-    if n_windows is None and not n_samples_window_is_integer:
-        type_n_samples_window = type(n_samples_window)
-        raise Exception('n_samples_window must be an integer. '
-                        'Now, the class of n_samples_window is {}'.format(
-            type_n_samples_window))
-
-    n_windows_is_integer = isinstance(n_windows, numbers.Integral)
-    if n_samples_window is None and not n_windows_is_integer:
-        type_n_windows = type(n_windows)
-        raise Exception('n_windows must be an integer. '
-                        'Now, the class of n_windows is {}'.format(
-            type_n_windows))
+    if axis > n_d_data:
+        raise Exception(
+            'shuffle in dimension {} can\'t be done,\n'
+            'because data has {} dimensions and\n'
+            'it doesn\'t have the {}th dimension.\n'
+            'axis has to be an integer in the interval\n'
+            '0 =< axis < n_d_data.\n'
+            'In this case, axis = {} and n_d_data = {}'.format(
+                axis, n_d_data, axis, axis, n_d_data))
 
     if n_samples_window is None and n_windows is None:
         raise Exception('Both n_samples_window and n_windows are None.'
                         'Define only one of them with an integer.'
                         'One of them has to be None and the other one has to be an integer')
 
-    if n_samples_window is None and n_windows_is_integer:
-        n_samples_window = ceil(n_samples_data / n_windows)
+    elif n_samples_window is None and n_windows is not None:
+        n_samples_window = math.ceil(n_samples_data / n_windows)
 
-    if n_windows is None and n_samples_window_is_integer:
-        n_windows = ceil(n_samples_data / n_samples_window)
+    elif n_samples_window is not None and n_windows is None:
+        n_windows = math.ceil(n_samples_data / n_samples_window)
 
-    if n_samples_window_is_integer and n_windows_is_integer:
+    elif n_samples_window is not None and n_windows is not None:
 
-        tmp_n_samples_window = ceil(n_samples_data / n_windows)
-        tmp_n_windows = ceil(n_samples_data / n_samples_window)
+        tmp_n_samples_window = math.ceil(n_samples_data / n_windows)
+        tmp_n_windows = math.ceil(n_samples_data / n_samples_window)
 
         if tmp_n_samples_window != n_samples_window and tmp_n_windows != n_windows:
 
-            raise Exception('For n_samples_data = {}, n_samples_window = {} and n_windows = {} are not compatible.\n'
-                            'If n_samples_window = {}, n_windows has to be {} or None.\n'
-                            'If n_windows = {}, n_samples_window has to be {} or None'.format(
-                n_samples_data, n_samples_window, n_windows, n_samples_window, tmp_n_windows, n_windows, tmp_n_samples_window))
-
-
-
+            raise Exception(
+                'For n_samples_data = {}, n_samples_window = {} and n_windows = {} are not compatible.\n'
+                'If n_samples_window = {}, n_windows has to be {} or None.\n'
+                'If n_windows = {}, n_samples_window has to be {} or None'.format(
+                    n_samples_data, n_samples_window, n_windows, n_samples_window,
+                    tmp_n_windows, n_windows, tmp_n_samples_window))
 
     if n_samples_window * n_windows > n_samples_data:
         print('Warning: n_samples_window * n_windows > n_samples_data.\n'
               'n_samples_window * n_windows = {} and n_samples_data = {}'.format(
             n_samples_window * n_windows, n_samples_data))
 
-    indexes_samples_i_window = np.empty(n_d_data, dtype=object)
+    indexes_i_window = np.empty(n_d_data, dtype=object)
     for i_d in range(n_d_data):
-        if i_d == dimension_shuffle:
-            # indexes_samples_i_window.append(slice(0, n_samples_window))
-            indexes_samples_i_window[i_d] = np.arange(n_samples_window)
-        else:
-            indexes_samples_i_window[i_d] = np.arange(shape_data[i_d])
-
-    # indexes_samples_i_window = np.array(indexes_samples_i_window)
-    # indexes_samples_i_window[dimension_shuffle] = indexes_samples_i_window[dimension_shuffle][indexes_samples_i_window[dimension_shuffle] < n_samples_data]
-    # indexes_samples_i_window_tuple = np.ix_(*indexes_samples_i_window)
-
-    tmp_shape_data_shuffle = list(shape_data)
-    tmp_shape_data_shuffle[dimension_shuffle] = 0
-    dtype_data = data.dtype
-    # data_shuffle = np.empty(tmp_shape_data_shuffle, dtype=dtype_data)
-    data_shuffle = np.empty(shape_data, dtype=dtype_data)
+        if i_d != axis:
+            indexes_i_window[i_d] = slice(0, shape_data[i_d], 1)
 
+    if dtype is None:
+        dtype = data.dtype
+    data_shuffle = np.empty(shape_data, dtype=dtype)
+    start = 0
+    end = n_samples_window
     for i_window in range(n_windows):
 
-        indexes_samples_i_window[dimension_shuffle] = indexes_samples_i_window[dimension_shuffle][indexes_samples_i_window[dimension_shuffle] < n_samples_data]
-        # indexes_samples_i_window = np.arange(n_samples_window * i_window, n_samples_window * (i_window + 1))
-
-        # indexes_samples_i_window = indexes_samples_i_window[indexes_samples_i_window < n_samples_data]
-        # index_tuple = tuple(index)
-        # index[dimension_shuffle] = start_i_window
-        # data = np.arange(6*6*6).reshape(6, 6, 6)
-        # i = np.ix_(*[[1, 3], [1, 3, 5], [0, 1, 3, 5]])
-        # if n_d_data == 1:
-        #     samples_i_window = data[np.ix_(indexes_samples_i_window)]
-        # else:
-        indexes_samples_i_window_tuple = np.ix_(*indexes_samples_i_window)
-            # samples_i_window = data[np.ix_(indexes_samples_i_window_tuple)]
-        samples_i_window = data[indexes_samples_i_window_tuple]
-            # samples_i_window = np.take(data, indexes_samples_i_window, axis=dimension_shuffle)
-
-        samples_i_window = shuffle_in_any_dimension(samples_i_window, dimension_shuffle=dimension_shuffle)
-        # data_shuffle = np.append(data_shuffle, samples_i_window, axis=dimension_shuffle)
-        data_shuffle[indexes_samples_i_window_tuple] = samples_i_window
-
-        # indexes_samples_i_window = np.array(indexes_samples_i_window_tuple)
-        # indexes_samples_i_window[dimension_shuffle] += n_samples_window
-        # indexes_samples_i_window[dimension_shuffle] = indexes_samples_i_window[dimension_shuffle][indexes_samples_i_window[dimension_shuffle] < n_samples_data]
-        # indexes_samples_i_window_tuple = tuple(indexes_samples_i_window)
-
-        if i_window < n_windows - 1:
-            indexes_samples_i_window[dimension_shuffle] += n_samples_window
-
+        indexes_i_window[axis] = slice(start, end, 1)
+        indexes_tuple = tuple(indexes_i_window)
+        data_shuffle[indexes_tuple] = shuffle_in_any_dimension(data[indexes_tuple], axis=axis)
+        start = end
+        end += n_samples_window
 
     return data_shuffle
 
@@ -414,25 +357,19 @@ def shuffle_in_any_dimension(data, axis=0):
         raise Exception('data must be an numpy array. '
                         'Now, the class of data is {}'.format(type_data))
 
-    dimension_shuffle_is_integer = isinstance(axis, numbers.Integral)
-    if not dimension_shuffle_is_integer:
-        type_dimension_shuffle = type(axis)
-        raise Exception('dimension_shuffle must be an integer. '
-                        'Now, the class of dimension_shuffle is {}'.format(
-            type_dimension_shuffle))
-
     shape_data = data.shape
     n_samples = shape_data[axis]
     n_axes = len(shape_data)
 
     if axis > n_axes:
-        raise Exception('shuffle in axis {} can\'t be done,\n'
-                        'because data has {} axes and\n'
-                        'it doesn\'t have the {}th axis.\n'
-                        'axis has to be an integer in the interval\n'
-                        '0 =< axis < n_axes.\n'
-                        'In this case, axis = {} and n_axes = {}'.format(
-            axis, n_axes, axis, axis, n_axes))
+        raise Exception(
+            'shuffle in axis {} can\'t be done,\n'
+            'because data has {} axes and\n'
+            'it doesn\'t have the {}th axis.\n'
+            'axis has to be an integer in the interval\n'
+            '0 =< axis < n_axes.\n'
+            'In this case, axis = {} and n_axes = {}'.format(
+                axis, n_axes, axis, axis, n_axes))
 
 
     indexes_new = np.empty(n_axes, dtype=object)
@@ -441,30 +378,79 @@ def shuffle_in_any_dimension(data, axis=0):
             # indexes_samples_i_window.append(slice(0, n_samples_window))
             indexes_new[a] = np.random.rand(n_samples).argsort()
         else:
-            indexes_new[a] = np.arange(shape_data[a])
+            indexes_new[a] = slice(0, shape_data[a], 1)
 
-    # indexes_new = []
-    # for i_d in range(n_d_data):
-    #     if i_d == dimension_shuffle:
-    #         # indexes_samples_i_window.append(slice(0, n_samples_window))
-    #         indexes_new.append(np.random.rand(n_samples).argsort())
-    #     else:
-    #         indexes_new.append(np.arange(0, shape_data[i_d]))
-    #
-    # indexes_new = np.array(indexes_new)
-
-    # indexes_new = np.random.rand(n_samples).argsort()
-    # data_shuffle = np.take(data, indexes_new, axis=dimension_shuffle, out=data)
-    # if n_d_data == 1:
-    #     data_shuffle = data[np.ix_(indexes_new)]
-    #
-    # else:
-    data_shuffle = data[advanced_indexing(indexes_new)]
+    data_shuffle = data[tuple(indexes_new)]
 
     return data_shuffle
 
 
-def conditions_of_txts_to_arrays(conditions_of_directories, rows, columns, dtype=None):
+def csv_file_to_array(filename, rows, columns, dtype=None):
+
+    with open(filename, newline='') as csvfile:
+        reader = csv.reader(csvfile, delimiter=',')
+        table = np.asarray(list(reader), dtype=dtype)
+
+    indexes = tuple(cc_Formatter.numeric_indexes_to_slice([rows, columns]))
+    array = table[indexes]
+
+    return array
+
+
+def csv_file_to_arrays(filename, rows, columns, dtype=None):
+    try:
+        n_arrays_from_rows = len(rows)
+    except TypeError:
+        rows = [rows]
+        n_arrays_from_rows = 1
+    try:
+        n_arrays_from_columns = len(columns)
+    except TypeError:
+        columns = [columns]
+        n_arrays_from_columns = 1
+
+    if n_arrays_from_rows != n_arrays_from_columns:
+        if n_arrays_from_rows == 1:
+            rows = list(rows) * n_arrays_from_columns
+            n_arrays_from_rows = n_arrays_from_columns
+        elif n_arrays_from_columns == 1:
+            columns = list(columns) * n_arrays_from_rows
+            n_arrays_from_columns = n_arrays_from_rows
+        else:
+            raise ValueError(
+                'The following assumption is not met:\n'
+                '\t n_arrays_from_rows' + ' \u003D ' + 'n_arrays_from_columns')
+
+    n_arrays = n_arrays_from_rows
+    try:
+        n_dtypes = len(dtype)
+    except TypeError:
+        dtype = [dtype]
+        n_dtypes = 1
+    if n_arrays != n_dtypes:
+        if n_dtypes == 1:
+            dtype = list(dtype) * n_arrays
+        else:
+            raise ValueError(
+                'The following assumption is not met:\n'
+                '\t(n_dtypes \u003D n_arrays_from_rows) OR (n_dtypes \u003D n_arrays_from_columns)')
+
+    with open(filename, newline='') as csvfile:
+        reader = csv.reader(csvfile, delimiter=',')
+        table = np.asarray(list(reader), dtype='U')
+
+    n_axes_per_text = 2
+    arrays = [None] * n_arrays
+    for a in range(n_arrays):
+        indexes = tuple(cc_Formatter.numeric_indexes_to_slice([rows[a], columns[a]]))
+        if (dtype[a] is None) or (dtype[a] == str) or ('U' in dtype[a][:3]):
+            arrays[a] = table[indexes]
+        else:
+            arrays[a] = table[indexes].astype(dtype[a])
+    return arrays
+
+
+def conditions_of_csv_files_to_arrays(conditions_of_directories, rows, columns, dtype=None):
 
     n_axes_directories = len(conditions_of_directories)
     n_conditions_directories = np.empty(n_axes_directories, dtype=int)
@@ -500,68 +486,78 @@ def conditions_of_txts_to_arrays(conditions_of_directories, rows, columns, dtype
                 '\t n_arrays_from_rows' + ' \u003D ' + 'n_arrays_from_columns')
 
     n_arrays = n_arrays_from_rows
-    # try:
-    #     n_dtypes = len(dtype)
-    # except TypeError:
-    #     dtype = [dtype]
-    #     n_dtypes = 1
-    # if n_arrays != n_dtypes:
-    #     if n_dtypes == 1:
-    #         dtype = list(dtype) * n_arrays
-    #     else:
-    #         raise ValueError(
-    #             'The following assumption is not met:\n'
-    #             '\t(n_dtypes \u003D n_arrays_from_rows) AND (n_dtypes \u003D n_arrays_from_columns)')
+    try:
+        n_dtypes = len(dtype)
+    except TypeError:
+        dtype = [dtype]
+        n_dtypes = 1
+    if n_arrays != n_dtypes:
+        if n_dtypes == 1:
+            dtype = list(dtype) * n_arrays
+        else:
+            raise ValueError(
+                'The following assumption is not met:\n'
+                '\t(n_dtypes \u003D n_arrays_from_rows) OR (n_dtypes \u003D n_arrays_from_columns)')
 
-    n_axes_per_text = 2
+    n_axes_per_csv = 2
     indexes = [None] * n_arrays
     arrays = [None] * n_arrays
     for a in range(n_arrays):
         indexes[a] = tuple(cc_Formatter.numeric_indexes_to_slice([rows[a], columns[a]]))
-        # indexes[a] = tuple(cc_Formatter.numeric_indexes_to_slice(indexes[a]))
 
-    directory_text_0 = os.path.join(*[
+    directory_csv_d = os.path.join(*[
         conditions_of_directories[i][0] for i in range(n_axes_directories)])
-    array_text_0 = np.genfromtxt(directory_text_0, delimiter=',', dtype=dtype)
-    if dtype is None:
-        dtype = array_text_0.dtype
-    # axes_squeezed = [None] * n_arrays
+
+    with open(directory_csv_d, newline='') as csvfile:
+        reader = csv.reader(csvfile, delimiter=',')
+        array_csv_d = np.asarray(list(reader), dtype='U')
+
     indexes_out = [None] * n_arrays
     for a in range(n_arrays):
-        array_a_d = array_text_0[indexes[a]]
+        if dtype[a] is None:
+            dtype[a] = 'U'
+
+        array_a_d = array_csv_d[indexes[a]]
         shape_array_a_d = np.asarray(array_a_d.shape, dtype=int)
         shape_array_a = np.append(
             n_conditions_directories[axes_directories_squeezed], shape_array_a_d)
-        # axes_squeezed[a] = shape_array_a > 1
-        # shape_array_a = shape_array_a[axes_squeezed[a]]
-        arrays[a] = np.empty(shape_array_a, dtype=dtype)
+
+        arrays[a] = np.empty(shape_array_a, dtype=dtype[a])
         n_axes_a = shape_array_a.size
-        indexes_out[a] = np.empty(n_axes_a, dtype=object)
-        indexes_out[a][:] = slice(None)
+        indexes_out = np.empty(n_axes_a, dtype=object)
+        indexes_out[:] = slice(None)
 
     for d in range(n_combinations_directories):
 
-        directory_text_d = os.path.join(*[
+        directory_csv_d = os.path.join(*[
             conditions_of_directories[i][combinations_directories[d, i]] for i in range(n_axes_directories)])
 
-        array_text_d = np.genfromtxt(directory_text_d, delimiter=',', dtype=dtype)
+        with open(directory_csv_d, newline='') as csvfile:
+            reader = csv.reader(csvfile, delimiter=',')
+            array_csv_d = np.asarray(list(reader), dtype='U')
+
+        indexes_out[slice(0, n_axes_directories_squeezed, 1)] = (
+            combinations_directories[d, axes_directories_squeezed])
+
         for a in range(n_arrays):
-
-            indexes_out[a][slice(0, n_axes_directories_squeezed, 1)] = (
-                combinations_directories[d, axes_directories_squeezed])
-
-            array_a_d = array_text_d[indexes[a]]
-
-            arrays[a][tuple(indexes_out[a])] = array_a_d
+            arrays[a][tuple(indexes_out)] = array_csv_d[indexes[a]]  # .astype(dtype[a])
 
     return arrays
 
 
-def n_csv_files_to_array(directories_csv_files, names_csv_files_in_directories=False, rows=slice(None), columns=slice(None), data_type=None):
+def lines_to_csv_file(directory, lines, headers=None):
+    with open(directory, 'w', newline='') as csv_file:
+        writer = csv.writer(csv_file, delimiter=',')
+        if headers is not None:
+            writer.writerow(headers)
+        writer.writerows(lines)
+
+
+def n_csv_files_to_array_old(directories_csv_files, names_csv_files_in_directories=False, rows=slice(None), columns=slice(None), data_type=None):
 
     print('using the funtion n_csv_files_to_array().\n'
           'In the future versions of ccalafiore, it will be removed.\n'
-          'Consider using conditions_of_txts_to_arrays()')
+          'Consider using conditions_of_csv_files_to_arrays()')
     array = None
 
     n_axes_directories = len(directories_csv_files)
@@ -607,10 +603,9 @@ def n_csv_files_to_array(directories_csv_files, names_csv_files_in_directories=F
 
             end_row_f_file = (f + 1) * shape_array_per_file[0]
 
-            indexes_array = tuple([
-                *combinations_directories[d][logical_indexes_conditions],
-                slice(start_row_f_file, end_row_f_file)
-            ])
+            indexes_array = tuple(
+                [*combinations_directories[d][logical_indexes_conditions],
+                 slice(start_row_f_file, end_row_f_file)])
 
             array[indexes_array] = array_per_file
             start_row_f_file = end_row_f_file
@@ -727,3 +722,153 @@ def pad_arr_1_from_arr_2(arr_1, arr_2, axis=0):
     arr_1 = arr_1[tuple(index_arr_1)]
 
     return arr_1
+
+
+def centered_indexes_to_normal_indexes(
+        centered_indexes, axis_indexes, shape_array):
+
+    axis_dimensions_of_array = int(not (axis_indexes == 1))
+
+    if isinstance(centered_indexes, list) or isinstance(centered_indexes, tuple):
+        centered_indexes = np.asarray(centered_indexes)
+        n_axes_indexes = len(centered_indexes.shape)
+    elif isinstance(centered_indexes, np.ndarray):
+        n_axes_indexes = len(centered_indexes.shape)
+    else:
+        n_axes_indexes = 0
+
+    if isinstance(shape_array, list) or isinstance(shape_array, tuple):
+        shape_array = np.asarray(shape_array)
+        n_axes_shape_array = shape_array.size
+    elif isinstance(shape_array, np.ndarray):
+        n_axes_shape_array = shape_array.size
+    else:
+        n_axes_shape_array = 0
+
+    if (n_axes_indexes == 2) and (n_axes_shape_array == 1):
+        shape_array = np.expand_dims(shape_array, axis=axis_indexes)
+
+    normal_indexes = (shape_array / 2) + centered_indexes
+    if isinstance(normal_indexes, np.ndarray):
+        normal_indexes = np.floor(normal_indexes)
+    else:
+        normal_indexes = math.floor(normal_indexes)
+
+    return normal_indexes
+
+
+class MyRange:
+    def __init__(self, raw_range):
+        if isinstance(raw_range, dict):
+            keys = ['start', 'stop', 'step']
+            for k in raw_range.keys():
+                if k in keys:
+                    continue
+                else:
+                    raise KeyError(k)
+            if raw_range.get('start') is None:
+                start = 0
+            else:
+                start = raw_range['start']
+
+            if raw_range.get('stop') is None:
+                raise ValueError("raw_range['stop']")
+            else:
+                stop = raw_range['stop']
+
+            if raw_range.get('step') is None:
+                step = 1
+            else:
+                step = raw_range['step']
+
+        elif isinstance(raw_range, range):
+            start = raw_range.start
+            stop = raw_range.stop
+            step = raw_range.step
+        elif isinstance(raw_range, (list, tuple, np.ndarray)):
+            n_elements = len(raw_range)
+            if n_elements == 1:
+                start = 0
+                stop = raw_range[0]
+                step = 1
+            elif n_elements == 2:
+                start, stop = raw_range
+                step = 1
+            elif n_elements == 3:
+                start, stop, step = raw_range
+            else:
+                raise ValueError('raw_range has to have at least 1 element. {} are given'.format(n_elements))
+        elif isinstance(raw_range, numbers.Number):
+            start = 0
+            stop = raw_range
+            step = 1
+        else:
+            raise TypeError('raw_range')
+
+        self.start = convert_to_int_or_float(start)
+        self.stop = convert_to_int_or_float(stop)
+        self.step = convert_to_int_or_float(step)
+        self.len = convert_to_int_or_float((self.stop - self.start) / self.step)
+
+    # def __init__(self, stop, *args):
+    #
+    #     n_args = len(args)
+    #     if n_args == 0:
+    #         self.start = 0
+    #         self.stop = stop
+    #         self.step = 1
+    #     elif n_args == 1:
+    #         self.start = stop
+    #         self.stop = args[0]
+    #         self.step = 1
+    #     elif n_args == 2:
+    #         self.start = stop
+    #         self.stop = args[0]
+    #         self.step = args[1]
+    #     else:
+    #         raise TypeError('MyRange expected at most 3 arguments, got {}'.format(n_args + 1))
+    #
+    #     self.len = convert_to_int_or_float((self.stop - self.start) / self.step)
+    #     # self.len = (self.stop - self.start) / self.step
+
+    def starts_of_n_within_equal_ranges(self, n):
+
+        len_each_equal_ranges = self.len / n
+        starts = np.arange(
+            start=self.start,
+            stop=self.stop - (len_each_equal_ranges * .1),
+            step=len_each_equal_ranges)
+        return starts
+
+    def stops_of_n_within_equal_ranges(self, n):
+
+        len_each_equal_ranges = self.len / n
+        stops = np.arange(
+            start=self.start + len_each_equal_ranges,
+            stop=self.stop + len_each_equal_ranges - (len_each_equal_ranges * .1),
+            step=len_each_equal_ranges)
+
+        return stops
+
+    def centers_of_n_within_equal_ranges(self, n):
+
+        len_each_equal_ranges = self.len / n
+        half_len_each_equal_ranges = len_each_equal_ranges / 2
+        centers = np.arange(
+            start=self.start + half_len_each_equal_ranges,
+            stop=self.stop + half_len_each_equal_ranges - (len_each_equal_ranges * .1),
+            step=len_each_equal_ranges)
+        return centers
+
+
+def read_image(directory, order='RGB'):
+    array_bgr = cv2.imread(directory, cv2.IMREAD_COLOR)
+    order_l = order.upper()
+    if order_l == 'RGB':
+        array_rgb = cv2.cvtColor(array_bgr, cv2.COLOR_BGR2RGB)
+        # array_rgb = array_bgr[:, :, slice(-1, -4, -1)]
+        return array_rgb
+    elif order_l == 'BGR':
+        return array_bgr
+    else:
+        raise ValueError('order')
